@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2020 Yves Renard, Julien Pommier.
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -18,13 +18,11 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_mesh_get.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id$
 #include <map>
 #include <getfemint_misc.h>
-#include <getfemint_mesh.h>
-#include <getfemint_convex_structure.h>
-#include <getfemint_pgt.h>
 #include <getfem/getfem_export.h>
+#include <getfem/getfem_mesh.h>
 
 using namespace getfemint;
 
@@ -46,8 +44,8 @@ get_structure_or_geotrans_of_convexes(const getfem::mesh& m,
   std::vector<id_type> ids; ids.reserve(cvlst.card());
   for (dal::bv_visitor cv(cvlst); !cv.finished(); ++cv) {
     if (class_id == CVSTRUCT_CLASS_ID)
-      ids.push_back(ind_convex_structure(m.structure_of_convex(cv)));
-    else ids.push_back(ind_pgt(m.trans_of_convex(cv)));
+      ids.push_back(store_cvstruct_object(m.structure_of_convex(cv)));
+    else ids.push_back(store_geotrans_object(m.trans_of_convex(cv)));
   }
   out.return_packed_obj_ids(ids, class_id);
 }
@@ -59,7 +57,7 @@ transform_edge_list(const getfem::mesh &m, unsigned N, const bgeot::edge_list &e
   bgeot::edge_list::const_iterator it;
   size_type cv = size_type(-1);
   getfem::mesh::ref_convex cv_ref;
-  bgeot::pgeometric_trans pgt = NULL;
+  bgeot::pgeometric_trans pgt;
   size_type ecnt = 0;
   for (it = el.begin(); it != el.end(); it++, ecnt++) {
     if (cv != (*it).cv) {
@@ -156,17 +154,19 @@ typedef dal::dynamic_tree_sorted<mesh_faces_by_pts_list_elt> mesh_faces_by_pts_l
 
 
 static void
-outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out, const std::string &condition="")
+outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out,
+            const std::string &condition="")
 {
   mesh_faces_by_pts_list lst;
   dal::bit_vector cvlst, checked_pids, rejected_pids;
 
   bool with_normal(condition == "direction");
   bool with_box(condition == "box");
+  bool with_ball(condition == "ball");
 
   darray normal_vector;
-  bgeot::base_node un, pmin, pmax;
-  scalar_type threshold(0);
+  bgeot::base_node un, pmin, pmax, center;
+  scalar_type threshold(0), radius(0);
   if (with_normal) {
     normal_vector = in.pop().to_darray();
     scalar_type angle = in.pop().to_scalar();
@@ -180,13 +180,24 @@ outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out, const std::
       pmin[k] = std::min(p1[k],p2[k]);
       pmax[k] = std::max(p1[k],p2[k]);
     }
+  } else if (with_ball) {
+    darray p1 = in.pop().to_darray(m.dim());
+    center.resize(m.dim());
+    for (size_type k=0; k < m.dim(); ++k) {
+      center[k] = p1[k];
+    }
+    radius = in.pop().to_scalar();
   }
 
+  dim_type elm_dim = m.dim();
+  if (in.remaining() && in.front().is_integer())
+   elm_dim = dim_type(in.pop().to_integer());
+
   if (in.remaining()) cvlst = in.pop().to_bit_vector(&m.convex_index());
   else cvlst = m.convex_index();
 
   for (dal::bv_visitor ic(cvlst); !ic.finished(); ++ic) {
-    if (m.structure_of_convex(ic)->dim() == m.dim()) {
+    if (m.structure_of_convex(ic)->dim() == elm_dim) {
       for (short_type f = 0; f < m.structure_of_convex(ic)->nb_faces(); f++) {
         bgeot::mesh_structure::ind_pt_face_ct pt
           = m.ind_points_of_face_of_convex(ic, f);
@@ -194,7 +205,7 @@ outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out, const std::
         size_type idx = lst.add_norepeat(mesh_faces_by_pts_list_elt(ic,f,p));
         lst[idx].cnt++;
       }
-    } 
+    }
 //     else { // DEPRECATED
 //       bgeot::mesh_structure::ind_cv_ct pt = m.ind_points_of_convex(ic);
 //       std::vector<size_type> p(pt.begin(), pt.end());
@@ -228,6 +239,27 @@ outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out, const std::
             break;
           }
         }
+      } else if (with_ball) {
+        for (std::vector<size_type>::const_iterator pid=lst[i].ptid.begin();
+             pid != lst[i].ptid.end(); ++pid) {
+          if (!checked_pids.is_in(*pid)) {
+            checked_pids.add(*pid);
+            scalar_type checked_radius = scalar_type(0.0);
+            for (size_type k=0; k < m.dim(); ++k) {
+              checked_radius += pow(m.points()[*pid][k] - center[k], 2);
+            }
+            checked_radius = std::sqrt(checked_radius);
+            for (size_type k=0; k < m.dim(); ++k)
+              if (checked_radius > radius) {
+                rejected_pids.add(*pid);
+                break;
+              }
+          }
+          if (rejected_pids.is_in(*pid)) {
+            lst[i].cnt = -1;
+            break;
+          }
+        }
       }
       if (lst[i].cnt == 1) fcnt++;
     }
@@ -246,6 +278,50 @@ outer_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out, const std::
 
 }
 
+static void 
+all_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out) {
+  dal::bit_vector cvlst;
+
+  if (in.remaining()) cvlst = in.pop().to_bit_vector(&m.convex_index());
+  else cvlst = m.convex_index();
+  getfem::mesh_region mr;
+  for (dal::bv_visitor ic(cvlst); !ic.finished(); ++ic) mr.add(ic);
+  getfem::mesh_region mrr =  all_faces_of_mesh(m, mr);
+
+  unsigned fcnt = 0;
+  for (getfem::mr_visitor i(mrr); !i.finished(); ++i) ++fcnt;
+  iarray w = out.pop().create_iarray(2, fcnt);
+  fcnt = 0;
+  for (getfem::mr_visitor i(mrr); !i.finished(); ++i) {
+    w(0,fcnt) = int(i.cv()+config::base_index());
+    w(1,fcnt) = int(short_type(i.f()+config::base_index()));
+    fcnt++;
+  }
+}
+
+
+
+static void 
+inner_faces(const getfem::mesh &m, mexargs_in &in, mexargs_out &out) {
+  dal::bit_vector cvlst;
+
+  if (in.remaining()) cvlst = in.pop().to_bit_vector(&m.convex_index());
+  else cvlst = m.convex_index();
+  getfem::mesh_region mr;
+  for (dal::bv_visitor ic(cvlst); !ic.finished(); ++ic) mr.add(ic);
+  getfem::mesh_region mrr =  inner_faces_of_mesh(m, mr);
+
+  unsigned fcnt = 0;
+  for (getfem::mr_visitor i(mrr); !i.finished(); ++i) ++fcnt;
+  iarray w = out.pop().create_iarray(2, fcnt);
+  fcnt = 0;
+  for (getfem::mr_visitor i(mrr); !i.finished(); ++i) {
+    w(0,fcnt) = int(i.cv()+config::base_index());
+    w(1,fcnt) = int(short_type(i.f()+config::base_index()));
+    fcnt++;
+  }
+}
+
 
 static bgeot::base_node
 normal_of_face(const getfem::mesh& mesh, size_type cv, short_type f, size_type node) {
@@ -273,23 +349,23 @@ struct sub_gf_mesh_set : virtual public dal::static_stored_object {
                    const getfem::mesh *pmesh) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_mesh_set> psub_command;
+typedef std::shared_ptr<sub_gf_mesh_set> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
 
 #define sub_command(name, arginmin, arginmax, argoutmin, argoutmax, code) { \
-    struct subc : public sub_gf_mesh_set {                                  \
-      virtual void run(getfemint::mexargs_in& in,                           \
-                       getfemint::mexargs_out& out,                         \
-                       const getfem::mesh *pmesh)                           \
-      { dummy_func(in); dummy_func(out); code }                             \
-    };                                                                      \
-    psub_command psubc = new subc;                                          \
-    psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;             \
-    psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;         \
-    subc_tab[cmd_normalize(name)] = psubc;                                  \
-  }                           
+    struct subc : public sub_gf_mesh_set {				\
+      virtual void run(getfemint::mexargs_in& in,			\
+                       getfemint::mexargs_out& out,			\
+                       const getfem::mesh *pmesh)			\
+      { dummy_func(in); dummy_func(out); code }				\
+    };									\
+    psub_command psubc = std::make_shared<subc>();			\
+    psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
+    psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
+    subc_tab[cmd_normalize(name)] = psubc;				\
+  }
 
 
 /*@GFDOC
@@ -345,7 +421,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
     points.@*/
     sub_command
       ("pts", 0, 1, 0, 1,
-       double nan = get_NaN();
+       double nan = ::nan("");
        if (!in.remaining()) {
          dal::bit_vector bv = pmesh->points().index();
          darray w = out.pop().create_darray(pmesh->dim(), unsigned(bv.last_true()+1));
@@ -385,7 +461,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET PIDs = ('pid in faces', @imat CVFIDs)
-    Search point #id listed in `CVFIDs`.
+    Return point #id listed in `CVFIDs`.
 
     `CVFIDs` is a two-rows matrix, the first row lists convex #ids,
     and the second lists face numbers. On return, `PIDs` is a
@@ -417,7 +493,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET PIDs = ('pid in cvids', @imat CVIDs)
-      Search point #id listed in `CVIDs`.
+      Return point #id listed in `CVIDs`.
       
       `PIDs` is a @MATLAB{row }vector containing points #id.@*/
     sub_command
@@ -437,7 +513,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET PIDs = ('pid in regions', @imat RIDs)
-    Search point #id listed in `RIDs`.
+    Return point #id listed in `RIDs`.
 
     `PIDs` is a @MATLAB{row }vector containing points #id.@*/
     sub_command
@@ -466,7 +542,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET PIDs = ('pid from coords', @mat PTS[, @scalar radius=0])
-    Search point #id whose coordinates are listed in `PTS`.
+    Return point #id whose coordinates are listed in `PTS`.
 
     `PTS` is an array containing a list of point coordinates. On
     return, `PIDs` is a @MATLAB{row }vector containing points
@@ -482,7 +558,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
        for (unsigned j=0; j < v.getn(); j++) {
          getfem::base_node P = v.col_to_bn(j);
          getfem::size_type id = size_type(-1);
-         if (!is_NaN(P[0])) id = pmesh->search_point(P,radius);
+         if (!std::isnan(P[0])) id = pmesh->search_point(P,radius);
          if (id == getfem::size_type(-1)) w[j] = -1;
          else w[j] = int(id + config::base_index());
        }
@@ -532,6 +608,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
     /*@GET @CELL{Pts, IDx} = ('pts from cvid'[, @imat CVIDs])
     Search point listed in `CVID`.
 
+    Return `Pts` and `IDx`.
     If `CVIDs` is omitted, all the convexes will be considered
     (equivalent to `CVIDs = MESH:GET('max cvid')`). `IDx` is a
     @MATLAB{row }vector, length(IDx) = length(CVIDs)+1. `Pts` is a
@@ -639,6 +716,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
     /*@GET [E,C] = ('curved edges', @int N [, CVLST])
     [OBSOLETE FUNCTION! will be removed in a future release]
 
+    Return E and C.
     More sophisticated version of MESH:GET('edges') designed for
     curved elements. This one will return N (N>=2) points of the
     (curved) edges. With N==2, this is equivalent to
@@ -665,7 +743,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET PIDs = ('orphaned pid')
-      Search point #id which are not linked to a convex.@*/
+      Return point #id which are not linked to a convex.@*/
     sub_command
       ("orphaned pid", 0, 0, 0, 1,
        dal::bit_vector bv = pmesh->points().index();
@@ -678,7 +756,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET CVIDs = ('cvid from pid', @ivec PIDs[, @bool share=False])
-    Search convex #ids related with the point #ids given in `PIDs`.
+    Return convex #ids related with the point #ids given in `PIDs`.
     
     If `share=False`, search convex whose vertex #ids are in `PIDs`.
     If `share=True`, search convex #ids that share the point #ids
@@ -745,51 +823,106 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
        );
 
 
-    /*@GET CVFIDs = ('outer faces'[, CVIDs])
-    Return the set of faces not shared by two convexes.
+    /*@GET CVFIDs = ('outer faces'[, dim][, CVIDs])
+    Return the set of faces not shared by two elements.
 
     The output `CVFIDs` is a two-rows matrix, the first row lists
     convex #ids, and the second one lists face numbers (local number
-    in the convex). If `CVIDs` is not given, all convexes are
-    considered, and it basically returns the mesh boundary. If `CVIDs`
+    in the convex). If `dim` is provided, the function is forced to
+    detect faces of elements that have dimension `dim`, e.g. `dim`=2 will
+    detect edges of surface elements, even if these belong to a 3D mesh.
+    If `CVIDs` is not given, all convexes are considered, and the
+    function basically returns the mesh boundary. If `CVIDs`
     is given, it returns the boundary of the convex set whose #ids are
     listed in `CVIDs`.@*/
     sub_command
-      ("outer faces", 0, 1, 0, 1,
+      ("outer faces", 0, 2, 0, 1,
        check_empty_mesh(pmesh);
        outer_faces(*pmesh, in, out);
        );
 
-    /*@GET CVFIDs = ('outer faces with direction', @vec v, @scalar angle [, CVIDs])
+    /*@GET CVFIDs = ('inner faces'[, CVIDs])
+    Return the set of faces shared at least by two elements in CVIDs.
+    Each face is represented only once and is arbitrarily chosen
+    between the two neighbor elements. @*/
+    sub_command
+      ("inner faces", 0, 1, 0, 1,
+       check_empty_mesh(pmesh);
+       inner_faces(*pmesh, in, out);
+       );
+    
+    /*@GET CVFIDs = ('all faces'[, CVIDs])
+    Return the set of faces of the in CVIDs (in all the mesh if CVIDs is
+    omitted). Note that the face shared by two neighbor elements will be
+    represented twice. @*/
+    sub_command
+      ("all faces", 0, 1, 0, 1,
+       check_empty_mesh(pmesh);
+       all_faces(*pmesh, in, out);
+       );
+
+    /*@GET CVFIDs = ('outer faces with direction', @vec v, @scalar angle[, dim][, CVIDs])
     Return the set of faces not shared by two convexes and with a mean outward vector lying within an angle `angle` (in radians) from vector `v`.
 
     The output `CVFIDs` is a two-rows matrix, the first row lists convex
     #ids, and the second one lists face numbers (local number in the
-    convex). If `CVIDs` is given, it returns portion of the boundary of
+    convex). The argument `dim` works as in outer_faces().
+    If `CVIDs` is given, it returns portion of the boundary of
     the convex set defined by the #ids listed in `CVIDs`.@*/
     sub_command
-      ("outer faces with direction", 2, 3, 0, 1,
+      ("outer faces with direction", 2, 4, 0, 1,
        check_empty_mesh(pmesh);
        outer_faces(*pmesh, in, out, "direction");
        );
 
     
-    /*@GET CVFIDs = ('outer faces in box', @vec pmin, @vec pmax [, CVIDs])
+    /*@GET CVFIDs = ('outer faces in box', @vec pmin, @vec pmax[, dim][, CVIDs])
     Return the set of faces not shared by two convexes and lying within the box defined by the corner points `pmin` and `pmax`.
 
     The output `CVFIDs` is a two-rows matrix, the first row lists convex
     #ids, and the second one lists face numbers (local number in the
-    convex). If `CVIDs` is given, it returns portion of the boundary of
+    convex). The argument `dim` works as in outer_faces().
+    If `CVIDs` is given, it returns portion of the boundary of
     the convex set defined by the #ids listed in `CVIDs`.@*/
     sub_command
-      ("outer faces in box", 2, 3, 0, 1,
+      ("outer faces in box", 2, 4, 0, 1,
        check_empty_mesh(pmesh);
        outer_faces(*pmesh, in, out, "box");
        );
 
-    
+    /*@GET CVFIDs = ('outer faces in ball', @vec center, @scalar radius[, dim][, CVIDs])
+    Return the set of faces not shared by two convexes and lying within the ball of corresponding `center` and `radius`.
+
+    The output `CVFIDs` is a two-rows matrix, the first row lists convex
+    #ids, and the second one lists face numbers (local number in the
+    convex). The argument `dim` works as in outer_faces().
+    If `CVIDs` is given, it returns portion of the boundary of
+    the convex set defined by the #ids listed in `CVIDs`.@*/
+    sub_command
+      ("outer faces in ball", 2, 4, 0, 1,
+       check_empty_mesh(pmesh);
+       outer_faces(*pmesh, in, out, "ball");
+       );
+
+    /*@GET CVFIDs = ('adjacent face', @int cvid, @int fid)
+    Return convex face of the neighbor element if it exists.
+    If the convex have more than one neighbor
+    relatively to the face ``f`` (think to bar elements in 3D for instance),
+    return the first face found. @*/
+    sub_command
+      ("adjacent face", 2, 2, 0, 1,
+       check_empty_mesh(pmesh);
+       size_type cv = in.pop().to_convex_number(*pmesh);
+       short_type f = in.pop().to_face_number(pmesh->structure_of_convex(cv)->nb_faces());
+       bgeot::convex_face cvf = pmesh->adjacent_face(cv, f);
+       getfem::mesh_region flst;
+       if (cvf.cv != size_type(-1))
+	 flst.add(cvf.cv,cvf.f);
+       out.pop().from_mesh_region(flst);
+       );
+
     /*@GET CVFIDs = ('faces from cvid'[, @ivec CVIDs][, 'merge'])
-    Return a list of convexes faces from a list of convex #id.
+    Return a list of convex faces from a list of convex #id.
 
     `CVFIDs` is a two-rows matrix, the first row lists convex #ids,
     and the second lists face numbers (local number in the convex).
@@ -815,10 +948,10 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
          for (short_type f=0; f < pmesh->structure_of_convex(cv)->nb_faces(); ++f) {
            bool add = true;
            if (merge) {
-             bgeot::mesh_structure::ind_set neighbours;
-             pmesh->neighbours_of_convex(cv, f, neighbours);
-             for (bgeot::mesh_structure::ind_set::const_iterator it = neighbours.begin();
-                  it != neighbours.end(); ++it) {
+             bgeot::mesh_structure::ind_set neighbors;
+             pmesh->neighbors_of_convex(cv, f, neighbors);
+             for (bgeot::mesh_structure::ind_set::const_iterator it = neighbors.begin();
+                  it != neighbors.end(); ++it) {
                if (*it < cv) { add = false; break; }
              }
            }
@@ -849,7 +982,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET N = ('normal of face', @int cv, @int f[, @int nfpt])
-    Evaluates the normal of convex `cv`, face `f` at the `nfpt` point of the face.
+    Return the normal vector of convex `cv`, face `f` at the `nfpt` point of the face.
 
     If `nfpt` is not specified, then the normal is evaluated at each
     geometrical node of the face.@*/
@@ -865,7 +998,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET N = ('normal of faces', @imat CVFIDs)
-    Evaluates (at face centers) the normals of convexes.
+    Return matrix of (at face centers) the normal vectors of convexes.
 
     `CVFIDs` is supposed a two-rows matrix, the first row lists convex
     #ids, and the second lists face numbers (local number in the convex).@*/
@@ -882,6 +1015,32 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
        );
 
 
+    /*@GET CVIDs = ('convexes in box', @vec pmin, @vec pmax)
+    Return the set of convexes lying entirely within the box defined by the corner points `pmin` and `pmax`.
+
+    The output `CVIDs` is a two-rows matrix, the first row lists convex
+    #ids, and the second one lists face numbers (local number in the
+    convex). If `CVIDs` is given, it returns portion of the boundary of
+    the convex set defined by the #ids listed in `CVIDs`.@*/
+    sub_command
+      ("convexes in box", 2, 2, 0, 1,
+       check_empty_mesh(pmesh);
+       int dim = pmesh->dim();
+       darray p1 = in.pop().to_darray(dim);
+       darray p2 = in.pop().to_darray(dim);
+       bgeot::base_node pmin(dim);
+       bgeot::base_node pmax(dim);
+       for (int k=0; k < dim; ++k) {
+         pmin[k] = std::min(p1[k],p2[k]);
+         pmax[k] = std::max(p1[k],p2[k]);
+       }
+       getfem::mesh_region mr = select_convexes_in_box(*pmesh, pmin, pmax);
+       iarray w = out.pop().create_iarray_h(unsigned(mr.size()));
+       size_type j(0);
+       for (getfem::mr_visitor i(mr); !i.finished(); ++i, ++j)
+         w[j] = int(i.cv()+config::base_index());
+       );
+
     /*@GET Q = ('quality'[, @ivec CVIDs])
     Return an estimation of the quality of each convex (:math:`0 \leq Q \leq 1`).@*/
     sub_command
@@ -1155,7 +1314,7 @@ void gf_mesh_get(getfemint::mexargs_in& m_in,
   }
 
   if (m_in.narg() < 2)  THROW_BADARG( "Wrong number of input arguments");
-  const getfem::mesh *pmesh = m_in.pop().to_const_mesh();
+  const getfem::mesh *pmesh = extract_mesh_object(m_in.pop());
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
