@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2000-2015 Julien Pommier
+ Copyright (C) 2000-2020 Julien Pommier
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -154,7 +154,7 @@ namespace bgeot {
     static_block_allocator() { if (!palloc) palloc=&dal::singleton<block_allocator,1000>::instance(); } //new block_allocator(); }
   };
   
-#if !defined GETFEM_HAVE_OPENMP
+#if !defined GETFEM_HAS_OPENMP
   /** container for small vectors of POD (Plain Old Data) types. Should be as fast as 
       std::vector<T> while beeing smaller and uses copy-on-write. The gain is especially
       valuable on 64 bits architectures.
@@ -174,9 +174,9 @@ namespace bgeot {
     typedef const T * const_iterator;
 
     reference operator[](size_type l)
-    { GMM_ASSERT2(l < size(), "out of range"); return base()[l]; }
+    { GMM_ASSERT2(l <=size(), "out of range, l="<<l<<"size="<<size()); return base()[l]; }
     value_type operator[](size_type l) const
-    { GMM_ASSERT2(l < size(), "out of range"); return const_base()[l]; }
+    { GMM_ASSERT2(l <= size(), "out of range, l="<<l<<"size="<<size()); return const_base()[l]; }
     value_type at(size_type l) const { return const_base()[l]; }
     iterator begin() { return base(); }
     const_iterator begin() const { return const_base(); }
@@ -236,7 +236,7 @@ namespace bgeot {
     small_vector<T> operator-() const 
     { return -1.*(*this); }
     small_vector<T> operator*(T v) const 
-    { return small_vector<T>(*this, std::bind2nd(std::multiplies<T>(),v)); }
+    {return small_vector<T>(*this, [&v](const auto &x) {return v * x;});}
     small_vector<T> operator/(T v) const { return (*this)*(T(1)/v); }
     small_vector<T>& operator+=(const small_vector<T>& other) {
       const_iterator b = other.begin(); iterator it = begin(); 
@@ -333,7 +333,7 @@ namespace bgeot {
     { return -1.*(*this); }
 
     small_vector<T> operator*(T v) const 
-    { return small_vector<T>(*this, std::bind2nd(std::multiplies<T>(),v)); }
+    {return small_vector<T>(*this, [&v](const auto &x) {return v * x;});}
 
     small_vector<T> operator/(T v) const { return (*this)*(T(1)/v); }
 
@@ -379,7 +379,7 @@ namespace bgeot {
     }
 
 
-#endif // #if !defined GETFEM_HAVE_OPENMP
+#endif // #if !defined GETFEM_HAS_OPENMP
 
 
   template<class T> std::ostream& operator<<(std::ostream& os, const small_vector<T>& v) {
@@ -394,7 +394,7 @@ namespace bgeot {
   template <class VEC_CONT>
   void vectors_to_base_matrix(base_matrix &G, const VEC_CONT &a) {
     size_type P = (*(a.begin())).size(), NP = a.end() - a.begin();
-    G.resize(P, NP);
+    G.base_resize(P, NP);
     typename VEC_CONT::const_iterator it = a.begin(), ite = a.end();
     base_matrix::iterator itm = G.begin();
     for (; it != ite; ++it, itm += P)
