@@ -1,11 +1,11 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2001-2015 Yves Renard, Julien Pommier
+ Copyright (C) 2001-2020 Yves Renard, Julien Pommier
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -72,20 +72,39 @@ namespace getfem {
     const stored_mesh_slice *psl;
     std::unique_ptr<mesh_fem> pmf;
     dal::bit_vector pmf_dof_used;
-    std::vector<unsigned> pmf_cell_type;
+    std::vector<unsigned> pmf_mapping_type;
     std::ofstream real_os;
     dim_type dim_;
     bool reverse_endian;
     enum { EMPTY, HEADER_WRITTEN, STRUCTURE_WRITTEN, IN_CELL_DATA,
            IN_POINT_DATA } state;
+
+    template<class T> void write_val(T v);
+    template<class V> void write_vec(V p, size_type qdim);
+    template<class IT> void write_3x3tensor(IT p);
+    void write_separ();
+
   public:
-    typedef enum { VTK_VERTEX = 1, VTK_LINE = 3, VTK_QUADRATIC_EDGE = 21,
-                   VTK_TRIANGLE = 5, VTK_QUADRATIC_TRIANGLE = 22,
-                   VTK_PIXEL = 8, VTK_QUAD = 9, VTK_QUADRATIC_QUAD = 23,
-                   VTK_TETRA = 10, VTK_QUADRATIC_TETRA = 24,
-                   VTK_WEDGE = 13, /*VTK_QUADRATIC_WEDGE = 26,*/
-                   VTK_VOXEL = 11, VTK_HEXAHEDRON = 12,
-                   VTK_QUADRATIC_HEXAHEDRON = 25 } vtk_cell_type;
+    typedef enum { VTK_VERTEX = 1,
+                   VTK_LINE = 3,
+                   VTK_TRIANGLE = 5,
+                   VTK_PIXEL = 8,
+                   VTK_QUAD = 9,
+                   VTK_TETRA = 10,
+                   VTK_VOXEL = 11,
+                   VTK_HEXAHEDRON = 12,
+                   VTK_WEDGE = 13,
+                   VTK_PYRAMID = 14,
+                   VTK_QUADRATIC_EDGE = 21,
+                   VTK_QUADRATIC_TRIANGLE = 22,
+                   VTK_QUADRATIC_QUAD = 23,
+                   VTK_QUADRATIC_TETRA = 24,
+                   VTK_QUADRATIC_HEXAHEDRON = 25,
+                   VTK_QUADRATIC_WEDGE = 26,
+                   VTK_QUADRATIC_PYRAMID = 27,
+                   VTK_BIQUADRATIC_QUAD = 28,
+                   VTK_TRIQUADRATIC_HEXAHEDRON = 29,
+                   VTK_BIQUADRATIC_QUADRATIC_WEDGE = 32 } vtk_cell_type;
     vtk_export(const std::string& fname, bool ascii_ = false);
     vtk_export(std::ostream &os_, bool ascii_ = false);
 
@@ -118,14 +137,14 @@ namespace getfem {
         'name' */
     template<class VECT> void write_sliced_point_data(const VECT& Uslice,
                                                       const std::string& name,
-                                                      getfem::size_type qdim=1);
+                                                      size_type qdim=1);
     /** export data which is constant over each element. You should not use
         this function if you are exporting a slice.  U should have
         convex_index().card() elements.  */
 
     template<class VECT> void write_cell_data(const VECT& U,
-                                              const std::string& name, 
-                                              getfem::size_type qdim = 1);
+                                              const std::string& name,
+                                              size_type qdim = 1);
     /** export a data_set correspounding to measures of quality for each convex
         of the supplied mesh (which should have the same number of convex than
         the one used in the vtk_export)
@@ -145,13 +164,9 @@ namespace getfem {
     void write_mesh_structure_from_mesh_fem();
     void switch_to_cell_data();
     void switch_to_point_data();
-    template<class T> void write_val(T v);
-    template<class V> void write_vec(V p, getfem::size_type qdim);
-    template<class IT> void write_3x3tensor(IT p);
-    void write_separ();
     template<class VECT> void write_dataset_(const VECT& U,
                                              const std::string& name,
-                                             getfem::size_type qdim,
+                                             size_type qdim,
                                              bool cell_data=false);
   };
 
@@ -166,7 +181,7 @@ namespace getfem {
     }
   }
 
-  template<class IT> void vtk_export::write_vec(IT p, getfem::size_type qdim) {
+  template<class IT> void vtk_export::write_vec(IT p, size_type qdim) {
     float v[3];
     for (size_type i=0; i < qdim; ++i) {
       v[i] = float(p[i]);
@@ -217,21 +232,21 @@ namespace getfem {
   }
 
   template<class VECT>
-  void vtk_export::write_cell_data(const VECT& U, const std::string& name, getfem::size_type qdim) {
+  void vtk_export::write_cell_data(const VECT& U, const std::string& name,
+                                   size_type qdim) {
     write_dataset_(U, name, qdim, true);
   }
 
   template<class VECT>
   void vtk_export::write_sliced_point_data(const VECT& U,
                                            const std::string& name,
-                                           getfem::size_type qdim) {
+                                           size_type qdim) {
     write_dataset_(U, name, qdim, false);
   }
 
   template<class VECT>
   void vtk_export::write_dataset_(const VECT& U, const std::string& name,
-                                  getfem::size_type qdim,
-                                  bool cell_data) {
+                                  size_type qdim, bool cell_data) {
     write_mesh();
     size_type nb_val = 0;
     if (cell_data) {
@@ -242,8 +257,8 @@ namespace getfem {
       switch_to_point_data();
       nb_val = psl ? psl->nb_points() : pmf_dof_used.card();
     }
-    //size_type Q = gmm::vect_size(U) / nb_val;
     size_type Q = qdim;
+    if (Q == 1) Q = gmm::vect_size(U) / nb_val;
     GMM_ASSERT1(gmm::vect_size(U) == nb_val*Q,
                 "inconsistency in the size of the dataset: "
                 << gmm::vect_size(U) << " != " << nb_val << "*" << Q);
@@ -558,13 +573,14 @@ namespace getfem {
 
   public:
     typedef enum {
-                   POS_PT = 0, //point
-                   POS_LN = 1, //line
-                   POS_TR = 2, //triangles
-                   POS_QU = 3, //quadrangles
-                   POS_SI = 4, //tetrahedra
-                   POS_HE = 5, //hexahedra
-                   POS_PR = 6  //prisms
+                   POS_PT = 0, // point
+                   POS_LN = 1, // line
+                   POS_TR = 2, // triangles
+                   POS_QU = 3, // quadrangles
+                   POS_SI = 4, // tetrahedra
+                   POS_HE = 5, // hexahedra
+                   POS_PR = 6, // prisms
+                   POS_PY = 7  // pyramids
     } pos_cell_types;
 
     pos_export(const std::string& fname);
@@ -657,7 +673,7 @@ namespace getfem {
   }
 
   template <class VECT>
-  void pos_export::write(const VECT& V, const size_type qdim_v){
+  void pos_export::write(const VECT& V, const size_type qdim_v) {
     int t;
     std::vector<unsigned> cell_dof;
     std::vector<scalar_type> cell_dof_val;
@@ -674,7 +690,7 @@ namespace getfem {
 
   template <class VECT>
   void pos_export::write_cell(const int& t, const std::vector<unsigned>& dof,
-                                            const VECT& val){
+                                            const VECT& val) {
     size_type qdim_cell = val.size()/dof.size();
     size_type dim3D = size_type(-1);
     if (1==qdim_cell){
@@ -695,6 +711,7 @@ namespace getfem {
       case POS_SI: os << "S("; break; // tetrahedra (simplex)
       case POS_HE: os << "H("; break; // hexahedra
       case POS_PR: os << "I("; break; // prism
+      case POS_PY: os << "Y("; break; // pyramid
     }
     for (size_type i=0; i<dof.size(); ++i){
       for(size_type j=0; j<dim; ++j){
