@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2004-2015 Yves Renard, Konstantinos Poulios.
+ Copyright (C) 2004-2020 Yves Renard, Konstantinos Poulios.
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -184,7 +184,7 @@ namespace getfem {
 
       size_type nb_vertices = gmm::mat_nrows(simplexes);
       std::vector<size_type> facet_vertices(nb_vertices);
-      std::vector< std::vector<size_type> > pt1_neighbours(size1);
+      std::vector< std::vector<size_type> > pt1_neighbors(size1);
       for (size_type i = 0; i < gmm::mat_ncols(simplexes); ++i) {
         gmm::copy(gmm::mat_col(simplexes, i), facet_vertices);
         for (size_type iv1 = 0; iv1 < nb_vertices-1; ++iv1) {
@@ -197,20 +197,20 @@ namespace getfem {
               bool already_in = false;
               size_type vv1 = (v1_on_surface1 ? v1 : v2);
               size_type vv2 = (v2_on_surface1 ? v1 : v2);
-              for (size_type j = 0; j < pt1_neighbours[vv1].size(); ++j)
-                if (pt1_neighbours[vv1][j] == vv2) {
+              for (size_type j = 0; j < pt1_neighbors[vv1].size(); ++j)
+                if (pt1_neighbors[vv1][j] == vv2) {
                   already_in = true;
                   break;
                 }
-              if (!already_in) pt1_neighbours[vv1].push_back(vv2);
+              if (!already_in) pt1_neighbors[vv1].push_back(vv2);
             }
           }
         }
       }
 
       for (size_type i1 = 0; i1 < size1; ++i1)
-        for (size_type j = 0; j < pt1_neighbours[i1].size(); ++j) {
-          size_type i2 = pt1_neighbours[i1][j] - size1;
+        for (size_type j = 0; j < pt1_neighbors[i1].size(); ++j) {
+          size_type i2 = pt1_neighbors[i1][j] - size1;
           size_type ii1 = size0 + i1;
           size_type ii2 = size0 + size1 + i2;
           contact_node *cn1 = &cnl1[i1];
@@ -318,7 +318,7 @@ namespace getfem {
         gmm::mult(grad_fc, gmm::transposed(base_ref_fc), grad_fc1);
         gmm::mult(G, grad_fc1, K);
         gmm::mult(gmm::transposed(K), K, CS);
-        gmm::lu_inverse(CS);
+        bgeot::lu_inverse(&(*(CS.begin())), P-1);
         gmm::mult(K, CS, B);
         gmm::mult(B, base_ref_fc, BB);
 
@@ -352,7 +352,7 @@ namespace getfem {
           pgt->poly_vector_grad(xp, grad_cv);
 
           base_matrix GG(N, nb_pts_cv);
-          vectors_to_base_matrix(GG, m.points_of_convex(cv));
+          m.points_of_convex(cv, GG);
 
           gmm::mult(GG, grad_cv, KK);
         }
@@ -416,7 +416,9 @@ namespace getfem {
         const mesh &mesh_m = cn_m->mf->linked_mesh();
         base_node slave_node = cn_s->mf->point_of_basic_dof(cn_s->dof);
         base_node master_node = cn_m->mf->point_of_basic_dof(cn_m->dof);
-        base_node un_sel(3), proj_node_sel(3), proj_node_ref_sel(3);
+        GMM_ASSERT1(slave_node.size() == qdim && master_node.size() == qdim,
+                    "Internal error");
+        base_node un_sel(qdim), proj_node_sel(qdim), proj_node_ref_sel(qdim);
         scalar_type is_in_min = 1e5;  //FIXME
         size_type cv_sel = 0;
         short_type fc_sel = 0;
@@ -424,7 +426,7 @@ namespace getfem {
         std::vector<short_type>::iterator fc;
         for (cv = cn_m->cvs.begin(), fc = cn_m->fcs.begin();
              cv != cn_m->cvs.end() && fc != cn_m->fcs.end(); cv++, fc++) {
-          base_node un(3), proj_node(3), proj_node_ref(3);
+          base_node un(qdim), proj_node(qdim), proj_node_ref(qdim);
           scalar_type is_in = projection_on_convex_face
             (mesh_m, *cv, *fc, master_node, slave_node, un, proj_node, proj_node_ref);
           if (is_in < is_in_min) {
@@ -439,8 +441,8 @@ namespace getfem {
         if (is_in_min < 0.05) {  //FIXME
           gap[row] = gmm::vect_sp(slave_node-proj_node_sel, un_sel);
 
-          base_node ut[3];
-          if (BT1) orthonormal_basis_to_unit_vec(d, un_sel, ut);
+	  std::vector<base_node> ut(d);
+          if (BT1) orthonormal_basis_to_unit_vec(d, un_sel, &(ut[0]));
 
           CONTACT_B_MATRIX *BN = 0;
           CONTACT_B_MATRIX *BT = 0;
@@ -473,7 +475,7 @@ namespace getfem {
           if (BN) {
             base_matrix G;
             base_matrix M(qdim, mf_disp->nb_basic_dof_of_element(cv_sel));
-            bgeot::vectors_to_base_matrix(G, mesh_m.points_of_convex(cv_sel));
+            mesh_m.points_of_convex(cv_sel, G);
             pfem pf = mf_disp->fem_of_element(cv_sel);
             bgeot::pgeometric_trans pgt = mesh_m.trans_of_convex(cv_sel);
             fem_interpolation_context
@@ -1203,6 +1205,12 @@ namespace getfem {
       is_init = false;
     }
 
+    void set_BN2(CONTACT_B_MATRIX &BN2_) {
+      gmm::resize(BN2, gmm::mat_nrows(BN2_), gmm::mat_ncols(BN2_));
+      gmm::copy(BN2_, BN2);
+      is_init = false;
+    }
+
     void set_DN(CONTACT_B_MATRIX &DN_) {
       gmm::resize(DN, gmm::mat_nrows(DN_), gmm::mat_ncols(DN_));
       gmm::copy(DN_, DN);
@@ -1293,10 +1301,10 @@ namespace getfem {
    std::string dataname_gap, std::string dataname_alpha,
    int aug_version, bool Hughes_stabilized) {
 
-    Coulomb_friction_brick *pbr_
-      = new Coulomb_friction_brick(aug_version, true, false, false, Hughes_stabilized);
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version, true, false, false, Hughes_stabilized);
+    pbrick pbr(pbr_);
     pbr_->set_BN1(BN);
-    pbrick pbr = pbr_;
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1325,6 +1333,55 @@ namespace getfem {
     return md.add_brick(pbr, vl, dl, tl, model::mimlist(), size_type(-1));
   }
 
+  //=========================================================================
+  //  Add a frictionless contact condition with BN, r, alpha given for
+  //  two deformable bodies
+  //=========================================================================
+
+  size_type add_basic_contact_brick_two_deformable_bodies
+  (model &md, const std::string &varname_u1, const std::string &varname_u2,
+   const std::string &multname_n,
+   const std::string &dataname_r, CONTACT_B_MATRIX &BN1, CONTACT_B_MATRIX &BN2,
+   std::string dataname_gap, std::string dataname_alpha,
+   int aug_version, bool Hughes_stabilized) {
+
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version, true, true, false, Hughes_stabilized);
+    pbrick pbr(pbr_);
+    pbr_->set_BN1(BN1);
+    pbr_->set_BN2(BN2);
+
+    model::termlist tl;
+    tl.push_back(model::term_description(varname_u1, varname_u1, false));
+    tl.push_back(model::term_description(varname_u2, varname_u2, false));
+    tl.push_back(model::term_description(varname_u1, multname_n, false));
+    tl.push_back(model::term_description(multname_n, varname_u1, false));
+    tl.push_back(model::term_description(varname_u2, multname_n, false));
+    tl.push_back(model::term_description(multname_n, varname_u2, false));
+    tl.push_back(model::term_description(multname_n, multname_n, false));
+    model::varnamelist dl(1, dataname_r);
+
+    if (dataname_gap.size() == 0) {
+      dataname_gap = md.new_name("contact_gap_on_" + varname_u1);
+      md.add_initialized_fixed_size_data
+        (dataname_gap, model_real_plain_vector(1, scalar_type(0)));
+    }
+    dl.push_back(dataname_gap);
+
+    if (dataname_alpha.size() == 0) {
+      dataname_alpha = md.new_name("contact_parameter_alpha_on_"+ multname_n);
+      md.add_initialized_fixed_size_data
+        (dataname_alpha, model_real_plain_vector(1, scalar_type(1)));
+    }
+    dl.push_back(dataname_alpha);
+
+    model::varnamelist vl(1, varname_u1);
+    vl.push_back(varname_u2);
+    vl.push_back(multname_n);
+
+    return md.add_brick(pbr, vl, dl, tl, model::mimlist(), size_type(-1));
+  }
+
 
   //=========================================================================
   //  Add a contact with friction condition with BN, r, alpha given.
@@ -1341,12 +1398,12 @@ namespace getfem {
 
     bool dynamic_terms = (dataname_gamma.size() > 0);
 
-    Coulomb_friction_brick *pbr_
-      = new Coulomb_friction_brick(aug_version,false, false,
-                            Tresca_version, Hughes_stabilized, dynamic_terms);
+    auto pbr_ = std::make_shared<Coulomb_friction_brick>
+      (aug_version,false, false, Tresca_version, Hughes_stabilized,
+       dynamic_terms);
+    pbrick pbr(pbr_);
     pbr_->set_BN1(BN);
     pbr_->set_BT1(BT);
-    pbrick pbr = pbr_;
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1594,8 +1651,8 @@ namespace getfem {
   (model &md, const mesh_im &mim, const std::string &varname_u,
    const std::string &multname_n, const std::string &dataname_r,
    size_type region, const std::string &obstacle, int aug_version) {
-    pbrick pbr
-      = new Coulomb_friction_brick_rigid_obstacle(aug_version, true, obstacle);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_rigid_obstacle>
+      (aug_version, true, obstacle);
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1621,8 +1678,8 @@ namespace getfem {
    const std::string &multname_n, const std::string &multname_t,
    const std::string &dataname_r, const std::string &dataname_friction_coeff,
    size_type region, const std::string &obstacle, int aug_version) {
-    pbrick pbr
-      = new Coulomb_friction_brick_rigid_obstacle(aug_version,false,obstacle);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_rigid_obstacle>
+      (aug_version, false, obstacle);
 
     model::termlist tl;
     tl.push_back(model::term_description(varname_u, varname_u, false));
@@ -1816,8 +1873,8 @@ namespace getfem {
 
     bool two_variables = (varname_u1.compare(varname_u2) != 0);
 
-    pbrick pbr = new Coulomb_friction_brick_nonmatching_meshes
-           (aug_version, true, two_variables, rg1, rg2, slave1, slave2);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_nonmatching_meshes>
+      (aug_version, true, two_variables, rg1, rg2, slave1, slave2);
 
     // Calculate multipliers size
     const mesh_fem &mf_u1 = md.mesh_fem_of_variable(varname_u1);
@@ -1887,8 +1944,8 @@ namespace getfem {
 
     bool two_variables = (varname_u1.compare(varname_u2) != 0);
 
-    pbrick pbr = new Coulomb_friction_brick_nonmatching_meshes
-           (aug_version, false, two_variables, rg1, rg2, slave1, slave2);
+    pbrick pbr = std::make_shared<Coulomb_friction_brick_nonmatching_meshes>
+	       (aug_version, false, two_variables, rg1, rg2, slave1, slave2);
 
     // Calculate multipliers size
     const mesh_fem &mf_u1 = md.mesh_fem_of_variable(varname_u1);
