@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2001-2015 Yves Renard
+ Copyright (C) 2001-2020 Yves Renard
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -31,35 +31,106 @@ namespace getfem
    * ------------------------------------------------------------- */
 
   struct gf2vtk_dof_mapping : public std::vector<std::vector<unsigned> > {};
+  struct gf2vtk_vtk_type : public std::vector<int> {};
 
-  static const std::vector<unsigned> &getfem_to_vtk_dof_mapping(int t) {
-    gf2vtk_dof_mapping &dm = dal::singleton<gf2vtk_dof_mapping>::instance();
-    if (dm.size() == 0) {
-      dm.resize(30);
-      bgeot::sc(dm[vtk_export::VTK_VERTEX]) = 0;
-      bgeot::sc(dm[vtk_export::VTK_LINE]) = 0, 1;
-      bgeot::sc(dm[vtk_export::VTK_QUADRATIC_EDGE]) = 0, 2, 1;
-      bgeot::sc(dm[vtk_export::VTK_TRIANGLE]) = 0, 1, 2;
-      bgeot::sc(dm[vtk_export::VTK_QUADRATIC_TRIANGLE]) = 0, 2, 5, 1, 4, 3;
-      bgeot::sc(dm[vtk_export::VTK_QUAD]) = 0, 1, 3, 2;
-      bgeot::sc(dm[vtk_export::VTK_PIXEL]) = 0, 1, 2, 3;
-      bgeot::sc(dm[vtk_export::VTK_QUADRATIC_QUAD]) = 0, 2, 8, 6, 1, 5, 7, 3;
-      bgeot::sc(dm[vtk_export::VTK_TETRA]) = 0, 1, 2, 3;
-      bgeot::sc(dm[vtk_export::VTK_QUADRATIC_TETRA]) = 0, 2, 5, 9, 1, 4, 3, 6, 7, 8;
-      bgeot::sc(dm[vtk_export::VTK_WEDGE]) = 0, 1, 2, 3, 4, 5;
-      //bgeot::sc(dm[vtk_export::VTK_QUADRATIC_WEDGE]) = 0, 6, 1, 7, 2, 8, 3, 4, 5;
-      bgeot::sc(dm[vtk_export::VTK_VOXEL]) = 0, 1, 2, 3, 4, 5, 6, 7;
-      bgeot::sc(dm[vtk_export::VTK_HEXAHEDRON]) = 0, 1, 3, 2, 4, 5, 7, 6;
-      bgeot::sc(dm[vtk_export::VTK_QUADRATIC_HEXAHEDRON]) = 0, 2, 8, 6, 18, 20, 26, 24, 1, 5, 7, 3, 19, 23, 25, 21, 9, 11, 17, 15;
-    }
-    return dm[t];
+  typedef enum { NO_VTK_MAPPING,
+                 N1_TO_VTK_VERTEX,
+                 N2_TO_VTK_LINE,
+                 N3_TO_VTK_TRIANGLE,
+                 N4_TO_VTK_PIXEL,
+                 N4_TO_VTK_QUAD,
+                 N4_TO_VTK_TETRA,
+                 N8_TO_VTK_VOXEL,
+                 N8_TO_VTK_HEXAHEDRON,
+                 N6_TO_VTK_WEDGE,
+                 N5_TO_VTK_PYRAMID,
+                 N3_TO_VTK_QUADRATIC_EDGE,
+                 N6_TO_VTK_QUADRATIC_TRIANGLE,
+                 N8_TO_VTK_QUADRATIC_QUAD,
+                 N10_TO_VTK_QUADRATIC_TETRA,
+                 N20_TO_VTK_QUADRATIC_HEXAHEDRON,
+                 N15_TO_VTK_QUADRATIC_WEDGE,
+                 N13_TO_VTK_QUADRATIC_PYRAMID,
+                 N14_TO_VTK_QUADRATIC_PYRAMID,
+                 N9_TO_VTK_BIQUADRATIC_QUAD,
+                 N27_TO_VTK_TRIQUADRATIC_HEXAHEDRON,
+                 N18_TO_VTK_BIQUADRATIC_QUADRATIC_WEDGE } vtk_mapping_type;
+
+
+  void init_gf2vtk() {
+    // see https://www.kitware.com/products/books/VTKUsersGuide.pdf
+    // and https://www.kitware.com/products/books/VTKTextbook.pdf
+    // (there are some conflicts between the two)
+    gf2vtk_dof_mapping &vtkmaps = dal::singleton<gf2vtk_dof_mapping>::instance();
+    gf2vtk_vtk_type &vtktypes = dal::singleton<gf2vtk_vtk_type>::instance();
+    vtkmaps.resize(25);
+    vtktypes.resize(25);
+
+    vtktypes[N1_TO_VTK_VERTEX] = vtk_export::VTK_VERTEX;
+    vtkmaps [N1_TO_VTK_VERTEX] = {0};
+    vtktypes[N2_TO_VTK_LINE] = vtk_export::VTK_LINE;
+    vtkmaps [N2_TO_VTK_LINE] = {0, 1};
+    vtktypes[N3_TO_VTK_TRIANGLE] = vtk_export::VTK_TRIANGLE;
+    vtkmaps [N3_TO_VTK_TRIANGLE] = {0, 1, 2};
+    vtktypes[N4_TO_VTK_PIXEL] = vtk_export::VTK_PIXEL;
+    vtkmaps [N4_TO_VTK_PIXEL] = {0, 1, 2, 3};
+    vtktypes[N4_TO_VTK_QUAD] = vtk_export::VTK_QUAD;
+    vtkmaps [N4_TO_VTK_QUAD] = {0, 1, 3, 2};
+    vtktypes[N4_TO_VTK_TETRA] = vtk_export::VTK_TETRA;
+    vtkmaps [N4_TO_VTK_TETRA] = {0, 1, 2, 3};
+    vtktypes[N8_TO_VTK_VOXEL] = vtk_export::VTK_VOXEL;
+    vtkmaps [N8_TO_VTK_VOXEL] = {0, 1, 2, 3, 4, 5, 6, 7};
+    vtktypes[N8_TO_VTK_HEXAHEDRON] = vtk_export::VTK_HEXAHEDRON;
+    vtkmaps [N8_TO_VTK_HEXAHEDRON] = {0, 1, 3, 2, 4, 5, 7, 6};
+    vtktypes[N6_TO_VTK_WEDGE] = vtk_export::VTK_WEDGE;
+    vtkmaps [N6_TO_VTK_WEDGE] = {0, 1, 2, 3, 4, 5};
+    vtktypes[N5_TO_VTK_PYRAMID] = vtk_export::VTK_PYRAMID;
+    vtkmaps [N5_TO_VTK_PYRAMID] = {0, 1, 3, 2, 4};
+    vtktypes[N3_TO_VTK_QUADRATIC_EDGE] = vtk_export::VTK_QUADRATIC_EDGE;
+    vtkmaps [N3_TO_VTK_QUADRATIC_EDGE] = {0, 2, 1};
+    vtktypes[N6_TO_VTK_QUADRATIC_TRIANGLE] = vtk_export::VTK_QUADRATIC_TRIANGLE;
+    vtkmaps [N6_TO_VTK_QUADRATIC_TRIANGLE] = {0, 2, 5, 1, 4, 3};
+    vtktypes[N8_TO_VTK_QUADRATIC_QUAD] = vtk_export::VTK_QUADRATIC_QUAD;
+    vtkmaps [N8_TO_VTK_QUADRATIC_QUAD] = {0, 2, 7, 5, 1, 4, 6, 3};
+    vtktypes[N10_TO_VTK_QUADRATIC_TETRA] = vtk_export::VTK_QUADRATIC_TETRA;
+    vtkmaps [N10_TO_VTK_QUADRATIC_TETRA] = {0, 2, 5, 9, 1, 4, 3, 6, 7, 8};
+    vtktypes[N20_TO_VTK_QUADRATIC_HEXAHEDRON] = vtk_export::VTK_QUADRATIC_HEXAHEDRON;
+    vtkmaps [N20_TO_VTK_QUADRATIC_HEXAHEDRON] = {0, 2, 7, 5, 12, 14, 19, 17, 1, 4, 6, 3, 13, 16, 18, 15, 8, 9, 11, 10};
+    vtktypes[N15_TO_VTK_QUADRATIC_WEDGE] = vtk_export::VTK_QUADRATIC_WEDGE;
+    vtkmaps [N15_TO_VTK_QUADRATIC_WEDGE] = {0, 2, 5, 9, 11, 14, 1, 4, 3, 10, 13, 12, 6, 7, 8};
+                                      // = {0, 5, 2, 9, 14, 11, 3, 4, 1, 12, 13, 10, 6, 8, 7};
+    vtktypes[N13_TO_VTK_QUADRATIC_PYRAMID] = vtk_export::VTK_QUADRATIC_PYRAMID;
+    vtkmaps [N13_TO_VTK_QUADRATIC_PYRAMID] = {0, 2, 7, 5, 12, 1, 4, 6, 3, 8, 9, 11, 10};
+    vtktypes[N14_TO_VTK_QUADRATIC_PYRAMID] = vtk_export::VTK_QUADRATIC_PYRAMID;
+    vtkmaps [N14_TO_VTK_QUADRATIC_PYRAMID] = {0, 2, 8, 6, 13, 1, 5, 7, 3, 9, 10, 12, 11};
+    vtktypes[N9_TO_VTK_BIQUADRATIC_QUAD] = vtk_export::VTK_BIQUADRATIC_QUAD;
+    vtkmaps [N9_TO_VTK_BIQUADRATIC_QUAD] = {0, 2, 8, 6, 1, 5, 7, 3, 4};
+    vtktypes[N27_TO_VTK_TRIQUADRATIC_HEXAHEDRON] = vtk_export::VTK_TRIQUADRATIC_HEXAHEDRON;
+    vtkmaps [N27_TO_VTK_TRIQUADRATIC_HEXAHEDRON] = {0, 2, 8, 6, 18, 20, 26, 24, 1, 5, 7, 3, 19, 23, 25, 21, 9, 11, 17, 15, 12, 14, 10, 16, 4, 22};
+    vtktypes[N18_TO_VTK_BIQUADRATIC_QUADRATIC_WEDGE] = vtk_export::VTK_BIQUADRATIC_QUADRATIC_WEDGE;
+    vtkmaps [N18_TO_VTK_BIQUADRATIC_QUADRATIC_WEDGE]  = {0, 2, 5, 12, 14, 17, 1, 4, 3, 13, 16, 15, 6, 8, 11, 7, 10, 9};
+  }
+
+  static const std::vector<unsigned> &
+  select_vtk_dof_mapping(unsigned t) {
+    gf2vtk_dof_mapping &vtkmaps = dal::singleton<gf2vtk_dof_mapping>::instance();
+    if (vtkmaps.size() == 0) init_gf2vtk();
+    return vtkmaps[t];
+  }
+
+  int select_vtk_type(unsigned t) {
+    gf2vtk_vtk_type &vtktypes = dal::singleton<gf2vtk_vtk_type>::instance();
+    if (vtktypes.size() == 0) init_gf2vtk();
+    return vtktypes[t];
   }
 
   vtk_export::vtk_export(std::ostream &os_, bool ascii_)
     : os(os_), ascii(ascii_) { init(); }
 
   vtk_export::vtk_export(const std::string& fname, bool ascii_)
-    : os(real_os), ascii(ascii_), real_os(fname.c_str()) {
+    : os(real_os), ascii(ascii_),
+    real_os(fname.c_str(), !ascii ? std::ios_base::binary | std::ios_base::out :
+                                    std::ios_base::out) {
     GMM_ASSERT1(real_os, "impossible to write to vtk file '" << fname << "'");
     init();
   }
@@ -132,8 +203,12 @@ namespace getfem
     dim_ = m.dim();
     GMM_ASSERT1(dim_ <= 3, "attempt to export a " << int(dim_)
               << "D slice (not supported)");
-    pmf.reset(new mesh_fem(const_cast<mesh&>(m),1));
-    pmf->set_classical_finite_element(1);
+    pmf = std::make_unique<mesh_fem>(const_cast<mesh&>(m), dim_type(1));
+    for (dal::bv_visitor cv(m.convex_index()); !cv.finished(); ++cv) {
+      bgeot::pgeometric_trans pgt = m.trans_of_convex(cv);
+      pfem pf = getfem::classical_fem(pgt, pgt->complexity() > 1 ? 2 : 1);
+      pmf->set_finite_element(cv, pf);
+    }
     exporting(*pmf);
   }
 
@@ -141,60 +216,86 @@ namespace getfem
     dim_ = mf.linked_mesh().dim();
     GMM_ASSERT1(dim_ <= 3, "attempt to export a " << int(dim_)
               << "D slice (not supported)");
-    if (&mf != pmf.get()) pmf.reset(new mesh_fem(mf.linked_mesh(),1));
-
+    if (&mf != pmf.get())
+      pmf = std::make_unique<mesh_fem>(mf.linked_mesh());
     /* initialize pmf with finite elements suitable for VTK (which only knows
-       isoparametric FEMs of order 1 and 2 (with inner points missing)) */
+       isoparametric FEMs of order 1 and 2) */
     for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
       pfem pf = mf.fem_of_element(cv);
 
-      bool discontinuous = false;
-      for (unsigned i=0; i < pf->nb_dof(cv); ++i) {
-        /* could be a better test for discontinuity .. */
-        if (!dof_linkable(pf->dof_types()[i])) { discontinuous = true; break; }
+      if (pf == fem_descriptor("FEM_Q2_INCOMPLETE(2)") ||
+          pf == fem_descriptor("FEM_Q2_INCOMPLETE(3)") ||
+          pf == fem_descriptor("FEM_PYRAMID_Q2_INCOMPLETE") ||
+          pf == fem_descriptor("FEM_PYRAMID_Q2_INCOMPLETE_DISCONTINUOUS") ||
+          pf == fem_descriptor("FEM_PRISM_INCOMPLETE_P2") ||
+          pf == fem_descriptor("FEM_PRISM_INCOMPLETE_P2_DISCONTINUOUS"))
+        pmf->set_finite_element(cv, pf);
+      else {
+        bool discontinuous = false;
+        for (unsigned i=0; i < pf->nb_dof(cv); ++i) {
+          /* could be a better test for discontinuity .. */
+          if (!dof_linkable(pf->dof_types()[i])) { discontinuous = true; break; }
+        }
+
+        pfem classical_pf1 = discontinuous ? classical_discontinuous_fem(pgt, 1)
+                                           : classical_fem(pgt, 1);
+
+        short_type degree = 1;
+        if ((pf != classical_pf1 && pf->estimated_degree() > 1) ||
+            pgt->structure() != pgt->basic_structure())
+          degree = 2;
+
+        pmf->set_finite_element(cv, discontinuous ?
+                                classical_discontinuous_fem(pgt, degree, 0, true) :
+                                classical_fem(pgt, degree, true));
       }
-      pfem classical_pf1 = discontinuous ? classical_discontinuous_fem(pgt, 1)
-                                        : classical_fem(pgt, 1);
-      short_type degree = short_type(((pf != classical_pf1 &&
-                                   pf->estimated_degree() > 1) ||
-                                  pgt->structure() !=
-                                  pgt->basic_structure()) ? 2 : 1);
-      pmf->set_finite_element(cv, discontinuous ?
-                              classical_discontinuous_fem(pgt, degree) :
-                              classical_fem(pgt, degree));
     }
     /* find out which dof will be exported to VTK */
 
     const mesh &m = pmf->linked_mesh();
-    pmf_cell_type.resize(pmf->convex_index().last_true() + 1, unsigned(-1));
+    pmf_mapping_type.resize(pmf->convex_index().last_true() + 1, unsigned(-1));
     pmf_dof_used.sup(0, pmf->nb_basic_dof());
     for (dal::bv_visitor cv(pmf->convex_index()); !cv.finished(); ++cv) {
-      int t = -1;
+      vtk_mapping_type t = NO_VTK_MAPPING;
       size_type nbd = pmf->fem_of_element(cv)->nb_dof(cv);
       switch (pmf->fem_of_element(cv)->dim()) {
-        case 0: t = VTK_VERTEX; break;
-        case 1:
-          if (nbd == 2) t = VTK_LINE;
-          else if (nbd == 3) t = VTK_QUADRATIC_EDGE; break;
-        case 2:
-          if (nbd == 3) t = VTK_TRIANGLE;
-          else if (nbd == 4) t = check_voxel(m.points_of_convex(cv)) ? VTK_PIXEL : VTK_QUAD;
-          else if (nbd == 6) t = VTK_QUADRATIC_TRIANGLE;
-          else if (nbd == 9) t = VTK_QUADRATIC_QUAD; break;
-        case 3:
-          if (nbd == 4) t = VTK_TETRA;
-          else if (nbd == 10) t = VTK_QUADRATIC_TETRA;
-          else if (nbd == 8) t = check_voxel(m.points_of_convex(cv)) ? VTK_VOXEL : VTK_HEXAHEDRON;
-          else if (nbd == 27) t = VTK_QUADRATIC_HEXAHEDRON;
-          else if (nbd == 6) t = VTK_WEDGE; break;
+      case 0: t = N1_TO_VTK_VERTEX; break;
+      case 1:
+        if (nbd == 2) t = N2_TO_VTK_LINE;
+        else if (nbd == 3) t = N3_TO_VTK_QUADRATIC_EDGE;
+        break;
+      case 2:
+        if (nbd == 3) t = N3_TO_VTK_TRIANGLE;
+        else if (nbd == 4)
+          t = check_voxel(m.points_of_convex(cv)) ? N4_TO_VTK_PIXEL
+                                                  : N4_TO_VTK_QUAD;
+        else if (nbd == 6) t = N6_TO_VTK_QUADRATIC_TRIANGLE;
+        else if (nbd == 8) t = N8_TO_VTK_QUADRATIC_QUAD;
+        else if (nbd == 9) t = N9_TO_VTK_BIQUADRATIC_QUAD;
+        break;
+      case 3:
+        if (nbd == 4) t = N4_TO_VTK_TETRA;
+        else if (nbd == 10) t = N10_TO_VTK_QUADRATIC_TETRA;
+        else if (nbd == 8)
+          t = check_voxel(m.points_of_convex(cv)) ? N8_TO_VTK_VOXEL
+                                                  : N8_TO_VTK_HEXAHEDRON;
+        else if (nbd == 20) t = N20_TO_VTK_QUADRATIC_HEXAHEDRON;
+        else if (nbd == 27) t = N27_TO_VTK_TRIQUADRATIC_HEXAHEDRON;
+        else if (nbd == 5) t = N5_TO_VTK_PYRAMID;
+        else if (nbd == 13) t = N13_TO_VTK_QUADRATIC_PYRAMID;
+        else if (nbd == 14) t = N14_TO_VTK_QUADRATIC_PYRAMID;
+        else if (nbd == 6) t = N6_TO_VTK_WEDGE;
+        else if (nbd == 15) t = N15_TO_VTK_QUADRATIC_WEDGE;
+        else if (nbd == 18) t = N18_TO_VTK_BIQUADRATIC_QUADRATIC_WEDGE;
+        break;
       }
-      GMM_ASSERT1(t != -1, "semi internal error.. could not map " <<
+      GMM_ASSERT1(t != -1, "semi internal error. Could not map " <<
                   name_of_fem(pmf->fem_of_element(cv))
                 << " to a VTK cell type");
-      pmf_cell_type[cv] = t;
+      pmf_mapping_type[cv] = t;
 
-      const std::vector<unsigned> &dmap = getfem_to_vtk_dof_mapping(t);
+      const std::vector<unsigned> &dmap = select_vtk_dof_mapping(t);
       //cout << "nbd = " << nbd << ", t = " << t << ", dmap = "<<dmap << "\n";
       GMM_ASSERT1(dmap.size() <= pmf->nb_basic_dof_of_element(cv),
                 "inconsistency in vtk_dof_mapping");
@@ -309,12 +410,12 @@ namespace getfem
 
     size_type nb_cell_values = 0;
     for (dal::bv_visitor cv(pmf->convex_index()); !cv.finished(); ++cv)
-      nb_cell_values += (1 + getfem_to_vtk_dof_mapping(pmf_cell_type[cv]).size());
+      nb_cell_values += (1 + select_vtk_dof_mapping(pmf_mapping_type[cv]).size());
 
     write_separ(); os << "CELLS " << pmf->convex_index().card() << " " << nb_cell_values << "\n";
 
     for (dal::bv_visitor cv(pmf->convex_index()); !cv.finished(); ++cv) {
-      const std::vector<unsigned> &dmap = getfem_to_vtk_dof_mapping(pmf_cell_type[cv]);
+      const std::vector<unsigned> &dmap = select_vtk_dof_mapping(pmf_mapping_type[cv]);
       write_val(int(dmap.size()));
       for (size_type i=0; i < dmap.size(); ++i)
         write_val(int(dofmap[pmf->ind_basic_dof_of_element(cv)[dmap[i]]]));
@@ -323,7 +424,7 @@ namespace getfem
 
     write_separ(); os << "CELL_TYPES " << pmf->convex_index().card() << "\n";
     for (dal::bv_visitor cv(pmf->convex_index()); !cv.finished(); ++cv) {
-      write_val(int(pmf_cell_type[cv]));
+      write_val(select_vtk_type(pmf_mapping_type[cv]));
       write_separ();
     }
 
@@ -447,26 +548,27 @@ namespace getfem
     dim_ = m.dim();
     GMM_ASSERT1(dim_ <= 3, "4D meshes and more are not supported");
     if (&mf != pmf.get())
-      pmf.reset(new mesh_fem(const_cast<mesh&>(m),1));
+      pmf = std::make_unique<mesh_fem>(const_cast<mesh&>(m), dim_type(1));
     bgeot::pgeometric_trans pgt = m.trans_of_convex(m.convex_index().first_true());
     GMM_ASSERT1(dxname_of_convex_structure
-              (pgt->structure()->basic_structure()) != 0,
+              (basic_structure(pgt->structure())) != 0,
               "DX Cannot handle " <<
               bgeot::name_of_geometric_trans(pgt) << ", use slices");
     /* initialize pmf with finite elements suitable for OpenDX */
     for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       bgeot::pgeometric_trans pgt2 = mf.linked_mesh().trans_of_convex(cv);
-      GMM_ASSERT1(pgt->structure()->basic_structure() ==
-                pgt2->structure()->basic_structure(),
-                "Cannot export this mesh to opendx, it contains "
-                "different convex types. Slice it first.");
+      GMM_ASSERT1(basic_structure(pgt->structure()) ==
+                  basic_structure(pgt2->structure()),
+                  "Cannot export this mesh to opendx, it contains "
+                  "different convex types. Slice it first.");
       pfem pf = mf.fem_of_element(cv);
       bool discontinuous = false;
       for (unsigned i=0; i < pf->nb_dof(cv); ++i) {
         /* could be a better test for discontinuity .. */
         if (!dof_linkable(pf->dof_types()[i])) { discontinuous = true; break; }
       }
-      pfem classical_pf1 = discontinuous ? classical_discontinuous_fem(pgt, 1) : classical_fem(pgt, 1);
+      pfem classical_pf1 = discontinuous ? classical_discontinuous_fem(pgt, 1)
+                                         : classical_fem(pgt, 1);
       pmf->set_finite_element(cv, classical_pf1);
     }
     pmf_dof_used.add(0, pmf->nb_basic_dof());
@@ -476,7 +578,7 @@ namespace getfem
   void dx_export::exporting(const mesh& m, std::string name) {
     dim_ = m.dim();
     GMM_ASSERT1(dim_ <= 3, "4D meshes and more are not supported");
-    pmf.reset(new mesh_fem(const_cast<mesh&>(m),1));
+    pmf = std::make_unique<mesh_fem>(const_cast<mesh&>(m), dim_type(1));
     pmf->set_classical_finite_element(1);
     exporting(*pmf, name);
   }
@@ -525,7 +627,8 @@ namespace getfem
   { strncpy(header, s.c_str(), 256); header[255] = 0; }
 
   void dx_export::check_header() {
-    if (header_written) return; header_written = true;
+    if (header_written) return;
+    header_written = true;
     os << "# data file for IBM OpenDX, generated by GetFem++ v "
        << GETFEM_VERSION << "\n";
     os << "# " << header << "\n";
@@ -713,7 +816,7 @@ namespace getfem
         write_val(int(pmf->ind_basic_dof_of_element(cv)[i]));
       write_separ();
     }
-    write_convex_attributes(pmf->linked_mesh().structure_of_convex(pmf->convex_index().first_true())->basic_structure());
+    write_convex_attributes(basic_structure(pmf->linked_mesh().structure_of_convex(pmf->convex_index().first_true())));
   }
 
   void dx_export::exporting_mesh_edges(bool with_slice_edges) {
@@ -779,14 +882,15 @@ namespace getfem
   static const std::vector<unsigned>& getfem_to_pos_dof_mapping(int t) {
     gf2pos_dof_mapping &dm = dal::singleton<gf2pos_dof_mapping>::instance();
     if (dm.size() == 0) {
-      dm.resize(7);
-      bgeot::sc(dm[pos_export::POS_PT]) = 0;
-      bgeot::sc(dm[pos_export::POS_LN]) = 0, 1;
-      bgeot::sc(dm[pos_export::POS_TR]) = 0, 1, 2;
-      bgeot::sc(dm[pos_export::POS_QU]) = 0, 1, 3, 2;
-      bgeot::sc(dm[pos_export::POS_SI]) = 0, 1, 2, 3;
-      bgeot::sc(dm[pos_export::POS_HE]) = 0, 1, 3, 2, 4, 5, 7, 6;
-      bgeot::sc(dm[pos_export::POS_PR]) = 0, 1, 2, 3, 4, 5;
+      dm.resize(8);
+      dm[pos_export::POS_PT] = {0};
+      dm[pos_export::POS_LN] = {0, 1};
+      dm[pos_export::POS_TR] = {0, 1, 2};
+      dm[pos_export::POS_QU] = {0, 1, 3, 2};
+      dm[pos_export::POS_SI] = {0, 1, 2, 3};
+      dm[pos_export::POS_HE] = {0, 1, 3, 2, 4, 5, 7, 6};
+      dm[pos_export::POS_PR] = {0, 1, 2, 3, 4, 5};
+      dm[pos_export::POS_PY] = {0, 1, 3, 2, 4};
     }
     return dm[t];
   }
@@ -802,7 +906,7 @@ namespace getfem
   }
 
   void pos_export::init() {
-    strcpy(header, "Exported by GetFEM++");
+    strcpy(header, "Exported by GetFEM");
     state = EMPTY;
     view  = 0;
   }
@@ -825,7 +929,7 @@ namespace getfem
     dim = dim_type(m.dim());
     GMM_ASSERT1(int(dim) <= 3, "attempt to export a "
                 << int(dim) << "D mesh (not supported)");
-    pmf.reset(new mesh_fem(const_cast<mesh&>(m),1));
+    pmf = std::make_unique<mesh_fem>(const_cast<mesh&>(m), dim_type(1));
     pmf->set_classical_finite_element(1);
     exporting(*pmf);
     state = STRUCTURE_WRITTEN;
@@ -836,7 +940,8 @@ namespace getfem
     dim = dim_type(mf.linked_mesh().dim());
     GMM_ASSERT1(int(dim) <= 3, "attempt to export a "
                 << int(dim) << "D mesh_fem (not supported)");
-    if (&mf != pmf.get()) pmf.reset(new mesh_fem(mf.linked_mesh(),1));
+    if (&mf != pmf.get())
+      pmf = std::make_unique<mesh_fem>(mf.linked_mesh(), dim_type(1));
 
     /* initialize pmf with finite elements suitable for Gmsh */
     for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
@@ -848,7 +953,8 @@ namespace getfem
         // could be a better test for discontinuity ...
         if (!dof_linkable(pf->dof_types()[i])) { discontinuous = true; break; }
       }
-      pfem classical_pf1 = discontinuous ? classical_discontinuous_fem(pgt, 1) : classical_fem(pgt, 1);
+      pfem classical_pf1 = discontinuous ?
+        classical_discontinuous_fem(pgt, 1) : classical_fem(pgt, 1);
       pmf->set_finite_element(cv, classical_pf1);
     }
     psl = NULL;
@@ -862,11 +968,13 @@ namespace getfem
         case 3: t = POS_TR; break;
         case 4:
           if ( 2 == pmf->fem_of_element(cv)->dim() ) t = POS_QU;
-          else if (3 == pmf->fem_of_element(cv)->dim()) t = POS_SI; break;
+          else if (3 == pmf->fem_of_element(cv)->dim()) t = POS_SI;
+          break;
         case 6: t = POS_PR; break;
         case 8: t = POS_HE; break;
+        case 5: t = POS_PY; break;
       }
-      GMM_ASSERT1(t != -1, "semi internal error.. could not map "
+      GMM_ASSERT1(t != -1, "semi internal error. Could not map "
                            << name_of_fem(pmf->fem_of_element(cv))
                            << " to a POS primitive type");
       pos_cell_type.push_back(unsigned(t));
@@ -936,7 +1044,7 @@ namespace getfem
     if (state >= IN_CELL_DATA) return;
     GMM_ASSERT1(int(m.dim()) <= 3, "attempt to export a "
                 << int(m.dim()) << "D mesh (not supported)");
-    pmf.reset(new mesh_fem(const_cast<mesh&>(m),1));
+    pmf = std::make_unique<mesh_fem>(const_cast<mesh&>(m), dim_type(1));
     pmf->set_classical_finite_element(1);
     write(*pmf,name);
     state = IN_CELL_DATA;
